;    awsqlitedb.pbi
;    Version 0.2 [2012/04/27]
;    Copyright (C) 2012 Ronny Krueger
;
;    This file is part of AWPB-Tools.
;
;    AWPB-Tools is free software: you can redistribute it and/or modify
;    it under the terms of the GNU Lesser General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    AWPB-Tools is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU Lesser General Public License for more details.
;
;    You should have received a copy of the GNU Lesser General Public License
;    along with AWPB-Tools.  If not, see <http://www.gnu.org/licenses/>.

XIncludeFile "awsqlstatement.pbi"

;{ Imports
CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Linux
    ImportC #PB_Compiler_Home + "purelibraries/linux/libraries/libpbsqlite3.a"
  CompilerCase #PB_OS_Windows
    ImportC "sqlite3.lib"
  CompilerEndSelect
  sqlite3_libversion()
  sqlite3_backup_init(*pDest, zDestName.p-ascii, *pSource, zSourceName.p-ascii) 
  sqlite3_backup_step(*sqlite3_backup, nPage)
  sqlite3_backup_finish(*sqlite3_backup)
  sqlite3_errcode(*db)
EndImport
;}
;{ SQLiteDB-Object
Interface AWPB_SQLiteDB
  Destroy             ()
  Query.b             (sqlcmd.s)
  Update.b            (sqlcmd.s)
  BeginTransaction.b  ()
  EndTransaction.b    ()
  CancelTransaction.b ()
  BeginBatch.b        ()
  Batch.b             (sqlcmd.s)
  EndBatch.b          (save.b)
  TableExists.b       (tablename.s)
  OpenDB.b            (filename.s, user.s, pass.s)
  OpenMemoryDB.b      ()
  CloseDB             ()
  Handle.i            ()
  GetString.s         (Column.i)
  GetQuad.q           (Column.i)
  GetLong.l           (Column.i)
  GetFloat.f          (Column.i)
  GetDouble.d         (Column.i)  
  NextRow.b           ()
  ColumnName.s        (Column.i)
  ColumnSize.i        (Column.i)
  ColumnType.i        (Column.i)
  Columns.i           ()
  DatabaseError.s     ()
  FinishQuery         ()
  FirstRow.b          ()
  PrepareStatement.i  (sqlcmd.s)
  IsReady.b           ()
  SaveMemoryDB.i			(filename.s)
  LoadMemoryDB.i			(filename.s)
EndInterface
Structure ____AWPB_SQLiteDB
  *vt.AWPB_SQLiteDB
  
  FileName.s
  Handle.i
  BatchActive.b
  TransactionActive.b
EndStructure
Procedure.i AWPB_CreateSQLiteDBObj()
  Protected *this.____AWPB_SQLiteDB
  
  *this = AllocateMemory(SizeOf(____AWPB_SQLiteDB))
  If *this 
    *this\vt                = ?____AWPB_SQLiteDB_VT
    *this\Handle            = #Null
    *this\TransactionActive = #False
    *this\BatchActive       = #False
    *this\FileName          = ""
    
    ProcedureReturn *this
  EndIf
  
  ProcedureReturn #Null
EndProcedure
Procedure.b ____AWPB_SQLiteDB_Query(*this.____AWPB_SQLiteDB, sqlcmd.s)
  If IsDatabase(*this\Handle)
		If DatabaseQuery(*this\Handle, sqlcmd)
			ProcedureReturn #True
		EndIf		
	EndIf
	ProcedureReturn #False		
EndProcedure
Procedure.b ____AWPB_SQLiteDB_Update(*this.____AWPB_SQLiteDB, sqlcmd.s)
  If IsDatabase(*this\Handle)
		If DatabaseUpdate(*this\Handle, sqlcmd)
			ProcedureReturn #True
		EndIf		
	EndIf
	ProcedureReturn #False  
EndProcedure
Procedure.b ____AWPB_SQLiteDB_BeginTransaction(*this.____AWPB_SQLiteDB)
  *this\TransactionActive = ____AWPB_SQLiteDB_Update(*this, "BEGIN TRANSACTION")
  ProcedureReturn *this\TransactionActive 
EndProcedure
Procedure.b ____AWPB_SQLiteDB_EndTransaction(*this.____AWPB_SQLiteDB)
  If *this\TransactionActive
    *this\TransactionActive = #False
    ProcedureReturn ____AWPB_SQLiteDB_Update(*this, "COMMIT TRANSACTION")
  EndIf
  
  ProcedureReturn #True
EndProcedure
Procedure.b ____AWPB_SQLiteDB_CancelTransaction(*this.____AWPB_SQLiteDB)
  If *this\TransactionActive
    *this\TransactionActive = #False
    ProcedureReturn ____AWPB_SQLiteDB_Update(*this, "ROLLBACK TRANSACTION")
  EndIf
  
  ProcedureReturn #True
EndProcedure
Procedure.b ____AWPB_SQLiteDB_BeginBatch(*this.____AWPB_SQLiteDB)
	*this\BatchActive = ____AWPB_SQLiteDB_BeginTransaction(*this)
	If Not *this\BatchActive
		MessageRequester("SQLite-Error", DatabaseError())
	EndIf
		
	ProcedureReturn *this\BatchActive
EndProcedure
Procedure.b ____AWPB_SQLiteDB_Batch(*this.____AWPB_SQLiteDB, sqlcmd.s)
  If Not *this\BatchActive : ProcedureReturn #False : EndIf
  If Not ____AWPB_SQLiteDB_Update(*this, sqlcmd)
    MessageRequester("SQLite-Error", DatabaseError())
    ProcedureReturn #False
  EndIf
	
	ProcedureReturn #True
EndProcedure
Procedure.b ____AWPB_SQLiteDB_EndBatch(*this.____AWPB_SQLiteDB, save.b)
  Protected res.b
  
  res = #True
  If *this\BatchActive
    *this\BatchActive = #False
    
		If save
			If Not ____AWPB_SQLiteDB_EndTransaction(*this)
			  MessageRequester("SQLite-Error", DatabaseError())
			  res = #False
			EndIf
		Else
			If Not ____AWPB_SQLiteDB_CancelTransaction(*this)
				MessageRequester("SQLite-Error", DatabaseError())
				res = #False
			EndIf
		EndIf
	EndIf
	
	ProcedureReturn res
EndProcedure
Procedure.b ____AWPB_SQLiteDB_TableExists(*this.____AWPB_SQLiteDB, tablename.s)
  Protected res.b
  
	res = #False
	If ____AWPB_SQLiteDB_Query(*this, "SELECT [sql] From sqlite_master WHERE [type] = 'table' And LOWER(name) = '" + LCase(TableName) + "' ")
	  If NextDatabaseRow(*this\Handle)
			res = #True
		EndIf
  EndIf
		
  ProcedureReturn res  
EndProcedure
Procedure   ____AWPB_SQLiteDB_CloseDB(*this.____AWPB_SQLiteDB)
	If IsDatabase(*this\Handle)
    If *this\BatchActive       : ____AWPB_SQLiteDB_EndBatch(*this, #False)  : EndIf
    If *this\TransactionActive : ____AWPB_SQLiteDB_CancelTransaction(*this) : EndIf
		CloseDatabase(*this\Handle)
	EndIf
	*this\Handle   					= #Null
	*this\FileName 					= ""
	*this\BatchActive 			= #False
	*this\TransactionActive = #False
EndProcedure
Procedure.b ____AWPB_SQLiteDB_OpenDB(*this.____AWPB_SQLiteDB, filename.s, user.s, pass.s)
	Protected f.i
	
	____AWPB_SQLiteDB_CloseDB(*this)
	
	*this\FileName = filename
	If CheckFilename(filename)
		Select FileSize(filename)
			Case -1:
				f = CreateFile(#PB_Any, filename)
				If IsFile(f)
					CloseFile(f)
				Else
					ProcedureReturn #False
				EndIf
			Case -2:
				ProcedureReturn #False
		EndSelect
	Else
		ProcedureReturn #False
	EndIf
	
	*this\Handle = OpenDatabase(#PB_Any, filename, user, pass, #PB_Database_SQLite)
	If IsDatabase(*this\Handle)
	  ProcedureReturn #True
	EndIf
	
	ProcedureReturn #False
EndProcedure
Procedure.b ____AWPB_SQLiteDB_OpenMemoryDB(*this.____AWPB_SQLiteDB)
	____AWPB_SQLiteDB_CloseDB(*this)
	
	*this\FileName = ":memory:"
	*this\Handle   = OpenDatabase(#PB_Any, *this\FileName, "", "", #PB_Database_SQLite)
	If IsDatabase(*this\Handle)
	  ProcedureReturn #True
	EndIf
	
	ProcedureReturn #False
EndProcedure
Procedure.i ____AWPB_SQLiteDB_Destroy(*this.____AWPB_SQLiteDB)
  ____AWPB_SQLiteDB_CloseDB(*this)

  FreeMemory(*this)
  ProcedureReturn #Null
EndProcedure
Procedure.i ____AWPB_SQLiteDB_Handle(*this.____AWPB_SQLiteDB)
  ProcedureReturn *this\Handle
EndProcedure
Procedure.s ____AWPB_SQLiteDB_GetString(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn GetDatabaseString(*this\Handle, Column)
EndProcedure
Procedure.q ____AWPB_SQLiteDB_GetQuad(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn GetDatabaseQuad(*this\Handle, Column)
EndProcedure
Procedure.l ____AWPB_SQLiteDB_GetLong(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn GetDatabaseLong(*this\Handle, Column)
EndProcedure
Procedure.f ____AWPB_SQLiteDB_GetFloat(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn GetDatabaseFloat(*this\Handle, Column)
EndProcedure
Procedure.d ____AWPB_SQLiteDB_GetDouble(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn GetDatabaseDouble(*this\Handle, Column)
EndProcedure
Procedure.b ____AWPB_SQLiteDB_NextRow(*this.____AWPB_SQLiteDB)
	If NextDatabaseRow(*this\Handle)
		ProcedureReturn #True
	EndIf
	ProcedureReturn #False
EndProcedure
Procedure.s ____AWPB_SQLiteDB_ColumnName(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn DatabaseColumnName(*this\Handle, Column)
EndProcedure
Procedure.i ____AWPB_SQLiteDB_ColumnSize(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn DatabaseColumnSize(*this\Handle, Column)
EndProcedure
Procedure.i ____AWPB_SQLiteDB_ColumnType(*this.____AWPB_SQLiteDB, Column.i)
  ProcedureReturn DatabaseColumnType(*this\Handle, Column)
EndProcedure
Procedure.i ____AWPB_SQLiteDB_Columns(*this.____AWPB_SQLiteDB)
  ProcedureReturn DatabaseColumns(*this\Handle)
EndProcedure
Procedure.s ____AWPB_SQLiteDB_DatabaseError(*this.____AWPB_SQLiteDB)
  ProcedureReturn DatabaseError()
EndProcedure
Procedure   ____AWPB_SQLiteDB_FinishQuery(*this.____AWPB_SQLiteDB)
  ProcedureReturn FinishDatabaseQuery(*this\Handle)
EndProcedure
Procedure.b ____AWPB_SQLiteDB_FirstRow(*this.____AWPB_SQLiteDB)
	If FirstDatabaseRow(*this\Handle)
		ProcedureReturn #True
	EndIf
	ProcedureReturn #False
EndProcedure
Procedure.i ____AWPB_SQLiteDB_PrepareStatement(*this.____AWPB_SQLiteDB, sqlcmd.s)
  Protected *stat.AWPB_SQLStatement
  
  *stat = AWPB_CreateSQLStatementObj()
  If *stat
    *stat\SetSQL(sqlcmd)
  EndIf
  
  ProcedureReturn *stat
EndProcedure
Procedure.b ____AWPB_SQLiteDB_IsReady(*this.____AWPB_SQLiteDB)
  If IsDatabase(*this\Handle)
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.i ____AWPB_SQLiteDB_SaveMemoryDB(*this.____AWPB_SQLiteDB, filename.s)
	Protected f, db, *pbackup, rc.i
	
	If Not IsDatabase(*this\Handle)
		ProcedureReturn 1 ; SQLITE_ERROR
	EndIf
		
	If CheckFilename(filename)
		If FileSize(filename) >= -1
			f = CreateFile(#PB_Any, filename)
			If IsFile(f)
				CloseFile(f)
			Else
				ProcedureReturn 1 ; SQLITE_ERROR
			EndIf
		EndIf
	Else
		ProcedureReturn 1 ; SQLITE_ERROR
	EndIf
	
	db = OpenDatabase(#PB_Any, filename, "", "", #PB_Database_SQLite)
	If Not IsDatabase(db)
	  ProcedureReturn 1 ; SQLITE_ERROR
	EndIf
	
	*pbackup = sqlite3_backup_init(DatabaseID(db), "main", DatabaseID(*this\Handle), "main")
	If *pbackup
		sqlite3_backup_step(*pbackup, -1)
		sqlite3_backup_finish(*pbackup)
  EndIf
  rc = sqlite3_errcode(DatabaseID(db))
  
  CloseDatabase(db)
  
  ProcedureReturn rc
EndProcedure
Procedure.i ____AWPB_SQLiteDB_LoadMemoryDB(*this.____AWPB_SQLiteDB, filename.s)
	Protected f, db, *pbackup, rc.i
	
	If Not ____AWPB_SQLiteDB_OpenMemoryDB(*this)
		ProcedureReturn 1 ; SQLITE_ERROR
	EndIf
		
	If CheckFilename(filename)
		If FileSize(filename) >= 0
			f = ReadFile(#PB_Any, filename)
			If IsFile(f)
				CloseFile(f)
			Else
				ProcedureReturn 1 ; SQLITE_ERROR
			EndIf
		EndIf
	Else
		ProcedureReturn 1 ; SQLITE_ERROR
	EndIf
	
	db = OpenDatabase(#PB_Any, filename, "", "", #PB_Database_SQLite)
	If Not IsDatabase(db)
	  ProcedureReturn 1 ; SQLITE_ERROR
	EndIf
	
	*pbackup = sqlite3_backup_init(DatabaseID(*this\Handle), "main", DatabaseID(db), "main")
	If *pbackup
		sqlite3_backup_step(*pbackup, -1)
		sqlite3_backup_finish(*pbackup)
  EndIf
  rc = sqlite3_errcode(DatabaseID(db))
  
  CloseDatabase(db)
  
  ProcedureReturn rc
EndProcedure

DataSection ;{
____AWPB_SQLiteDB_VT:
  Data.i @____AWPB_SQLiteDB_Destroy()
  Data.i @____AWPB_SQLiteDB_Query()
  Data.i @____AWPB_SQLiteDB_Update()
  Data.i @____AWPB_SQLiteDB_BeginTransaction()
  Data.i @____AWPB_SQLiteDB_EndTransaction()
  Data.i @____AWPB_SQLiteDB_CancelTransaction()
  Data.i @____AWPB_SQLiteDB_BeginBatch()
  Data.i @____AWPB_SQLiteDB_Batch()
  Data.i @____AWPB_SQLiteDB_EndBatch()
  Data.i @____AWPB_SQLiteDB_TableExists()
  Data.i @____AWPB_SQLiteDB_OpenDB()
  Data.i @____AWPB_SQLiteDB_OpenMemoryDB()
  Data.i @____AWPB_SQLiteDB_CloseDB()
  Data.i @____AWPB_SQLiteDB_Handle()
  Data.i @____AWPB_SQLiteDB_GetString()
  Data.i @____AWPB_SQLiteDB_GetQuad()
  Data.i @____AWPB_SQLiteDB_GetLong()
  Data.i @____AWPB_SQLiteDB_GetFloat()
  Data.i @____AWPB_SQLiteDB_GetDouble()
  Data.i @____AWPB_SQLiteDB_NextRow()
  Data.i @____AWPB_SQLiteDB_ColumnName()
  Data.i @____AWPB_SQLiteDB_ColumnSize()
  Data.i @____AWPB_SQLiteDB_ColumnType()
  Data.i @____AWPB_SQLiteDB_Columns()
  Data.i @____AWPB_SQLiteDB_DatabaseError()
  Data.i @____AWPB_SQLiteDB_FinishQuery()
  Data.i @____AWPB_SQLiteDB_FirstRow()
  Data.i @____AWPB_SQLiteDB_PrepareStatement()
  Data.i @____AWPB_SQLiteDB_IsReady()
  Data.i @____AWPB_SQLiteDB_SaveMemoryDB()
  Data.i @____AWPB_SQLiteDB_LoadMemoryDB()  
EndDataSection ;}
;} 
;{ Global Functions
Procedure.s AWPB_SQLiteLibVersion()
  Protected v.s
  
  v = PeekS(sqlite3_libversion(), #PB_Any, #PB_Ascii)
  ProcedureReturn v
EndProcedure
;}
 
 